﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace WebForms
{

    public class MyCompositeControl : CompositeControl
    {


        public string DefaultOperation { get; set; } = "+";

        protected List<Tuple<string, double>> History
        {
            get
            {
                return ViewState["History"] as List<Tuple<string, double>>;
            }
            set
            {
                ViewState["History"] = value;
            }
        }


        private TextBox Num1Txt { get; set; }
        private RequiredFieldValidator Num1Validator { get; set; }

        private TextBox Num2Txt { get; set; }
        private RequiredFieldValidator Num2Validator { get; set; }


        private DropDownList OperationSelect { get; set; }
        private Button ComputeBtn { get; set; }
        private Label ResultTxt { get; set; }



        protected override void CreateChildControls()
        {
            this.Num1Txt = new TextBox()
            {
                ID = "Num1Txt",
                CssClass = "form-control"
            };
            this.Num1Validator = new RequiredFieldValidator()
            {
                ControlToValidate = "Num1Txt",
                ErrorMessage = "Musíte zadat první číslo.",
                Display = ValidatorDisplay.Dynamic,
                ValidationGroup = "gr2"
            };

            this.Num2Txt = new TextBox()
            {
                ID = "Num2Txt",
                CssClass = "form-control"
            };
            this.Num2Validator = new RequiredFieldValidator()
            {
                ControlToValidate = "Num2Txt",
                ErrorMessage = "Musíte zadat druhé číslo.",
                Display = ValidatorDisplay.Dynamic,
                ValidationGroup = "gr2"
            };



            this.OperationSelect = new DropDownList()
            {
                CssClass = "form-control"
            };
            
            List<ListItem> options = new List<ListItem>() {
                new ListItem("+", "+"),
                new ListItem("-", "-"),
                new ListItem("*", "*"),
                new ListItem("/", "/")
            };

            this.OperationSelect.DataSource = options;
            this.OperationSelect.DataBind();

            this.OperationSelect.SelectedValue = this.DefaultOperation;




            this.ComputeBtn = new Button() {
                Text = "Spočítej",
                CssClass = "btn btn-success",
                ValidationGroup = "gr2"
            };
            this.ComputeBtn.Click += ComputeBtn_Click;
            this.ResultTxt = new Label();


            // Musíme přidat komponenty jako potomky - jinak o nich nebude WebForms vědět!
            this.Controls.Add(this.Num1Txt);
            this.Controls.Add(this.Num1Validator);
            this.Controls.Add(this.Num2Txt);
            this.Controls.Add(this.Num2Validator);
            this.Controls.Add(this.OperationSelect);
            this.Controls.Add(this.ComputeBtn);
            this.Controls.Add(this.ResultTxt);

        }

        private void ComputeBtn_Click(object sender, EventArgs e)
        {
            Page.Validate("gr2");
            if (Page.IsValid)
            {
                double val1 = double.Parse(this.Num1Txt.Text);
                double val2 = double.Parse(this.Num2Txt.Text);
                string operation = this.OperationSelect.SelectedValue;

                double result = 0;
                switch (operation)
                {
                    case "+":
                        result = val1 + val2;
                        break;
                    case "-":
                        result = val1 - val2;
                        break;
                    case "*":
                        result = val1 * val2;
                        break;
                    case "/":
                        result = val1 / val2;
                        break;
                }

                this.ResultTxt.Text = result.ToString();

                if (this.History == null) { this.History = new List<Tuple<string, double>>(); }
                this.History.Add(new Tuple<string, double>(val1 + operation + val2, result));

            }
        }

        protected override void RenderContents(HtmlTextWriter output)
        {


            output.Write("<div class=\"container\">");

            output.Write("<h1>Moje kalkulačka</h1>");


            output.Write("<div class=\"form-inline\">");

            this.Num1Txt.RenderControl(output);
            this.Num1Validator.RenderControl(output);
            this.OperationSelect.RenderControl(output);
            this.Num2Txt.RenderControl(output);
            this.Num2Validator.RenderControl(output);
            this.ComputeBtn.RenderControl(output);


            output.Write("</div>");

            output.Write("<hr />");

            output.Write("<strong>Aktuální výsledek:</strong>");

            this.ResultTxt.RenderControl(output);




            if (History != null)
            {
                output.Write("<table class=\"table\">");
                output.Write("<tr>");
                output.Write("<th>Zadání</th>");
                output.Write("<th>Výsledek</th>");
                output.Write("</tr>");
                foreach (var item in History)
                {
                    output.Write("<tr>");
                    output.Write("<td>" + item.Item1 + "</td>");
                    output.Write("<td>" + item.Item2 + "</td>");
                    output.Write("</tr> ");
                }
                output.Write("</table>");
            }

            output.Write("</div>");


        }
    }
}
